@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import math
+import traceback
 from contextlib import suppress
 from copy import copy
 from typing import Dict, List, Any, Optional, Union, Callable
@@ -13,9 +14,11 @@ from OpenGL.raw.GL.VERSION.GL_1_0 import glEnable, GL_TEXTURE_2D, GL_DEPTH_TEST,
     glDisable, GL_CULL_FACE, GL_BACK, glCullFace, GL_DEPTH_COMPONENT
 from OpenGL.raw.GL.VERSION.GL_1_2 import GL_UNSIGNED_INT_8_8_8_8, GL_BGRA
 from glm import mat4, vec3, quat, vec4
+from pykotor.resource.generics.uti import read_uti
+
 from pykotor.common.geometry import Vector2, Vector3
 
-from pykotor.common.misc import CaseInsensitiveDict
+from pykotor.common.misc import CaseInsensitiveDict, EquipmentSlot
 from pykotor.common.module import Module
 from pykotor.common.stream import BinaryReader
 from pykotor.extract.file import ResourceIdentifier
@@ -25,6 +28,7 @@ from pykotor.resource.formats.tpc import TPC
 from pykotor.resource.formats.twoda import read_2da, TwoDA
 from pykotor.resource.generics.git import GIT, GITPlaceable, GITCreature, GITDoor, GITTrigger, GITEncounter, \
     GITWaypoint, GITSound, GITStore, GITCamera, GITInstance
+from pykotor.resource.generics.utc import UTC
 from pykotor.resource.type import ResourceType
 
 from pykotor.gl.shader import Shader, KOTOR_VSHADER, KOTOR_FSHADER, Texture, PICKER_FSHADER, PICKER_VSHADER, \
@@ -35,6 +39,7 @@ from pykotor.gl.models.predefined_mdl import STORE_MDL_DATA, STORE_MDX_DATA, WAY
     SOUND_MDL_DATA, SOUND_MDX_DATA, CAMERA_MDL_DATA, CAMERA_MDX_DATA, TRIGGER_MDL_DATA, TRIGGER_MDX_DATA, \
     ENCOUNTER_MDL_DATA, ENCOUNTER_MDX_DATA, ENTRY_MDL_DATA, ENTRY_MDX_DATA, EMPTY_MDL_DATA, EMPTY_MDX_DATA, \
     CURSOR_MDX_DATA, CURSOR_MDL_DATA, UNKNOWN_MDL_DATA, UNKNOWN_MDX_DATA
+from pykotor.tools import creature
 
 SEARCH_ORDER_2DA = [SearchLocation.OVERRIDE, SearchLocation.CHITIN]
 SEARCH_ORDER = [SearchLocation.CUSTOM_MODULES, SearchLocation.OVERRIDE, SearchLocation.CHITIN]
@@ -52,7 +57,7 @@ class Scene:
         self.installation: Optional[Installation] = installation
         self.textures: CaseInsensitiveDict[Texture] = CaseInsensitiveDict()
         self.models: CaseInsensitiveDict[Model] = CaseInsensitiveDict()
-        self.objects: List[RenderObject] = []
+        self.objects: Dict[Any, RenderObject] = {}
         self.selection: List[RenderObject] = []
         self.module: Optional[Module] = module
         self.camera: Camera = Camera()
@@ -62,7 +67,6 @@ class Scene:
 
         self.git: Optional[GIT] = None
         self.layout: Optional[LYT] = None
-        self.objects: Dict[Any, RenderObject] = {}
         self.clearCacheBuffer: List[ResourceIdentifier] = []
 
         self.picker_shader: Shader = Shader(PICKER_VSHADER, PICKER_FSHADER)
@@ -75,6 +79,7 @@ class Scene:
         self.table_placeables = TwoDA()
         self.table_creatures = TwoDA()
         self.table_heads = TwoDA()
+        self.table_baseitems = TwoDA()
         if installation is not None:
             self.setInstallation(installation)
 
@@ -99,6 +104,67 @@ class Scene:
         self.table_placeables = read_2da(installation.resource("placeables", ResourceType.TwoDA, SEARCH_ORDER_2DA).data)
         self.table_creatures = read_2da(installation.resource("appearance", ResourceType.TwoDA, SEARCH_ORDER_2DA).data)
         self.table_heads = read_2da(installation.resource("heads", ResourceType.TwoDA, SEARCH_ORDER_2DA).data)
+        self.table_baseitems = read_2da(installation.resource("baseitems", ResourceType.TwoDA, SEARCH_ORDER_2DA).data)
+
+    def getCreatureRenderObject(self, instance: GITCreature, utc: Optional[UTC] = None) -> RenderObject:
+        try:
+            if utc is None:
+                utc = self.module.creature(instance.resref.get()).resource()
+
+            head_obj = None
+            mask_hook = None
+
+            body_model, body_texture = creature.get_body_model(
+                utc, self.installation, appearance=self.table_creatures, baseitems=self.table_baseitems
+            )
+            head_model, head_texture = creature.get_head_model(
+                utc, self.installation, appearance=self.table_creatures, heads=self.table_heads
+            )
+            rhand_model, lhand_model = creature.get_weapon_models(
+                utc, self.installation, appearance=self.table_creatures, baseitems=self.table_baseitems
+            )
+            mask_model = creature.get_mask_model(
+                utc, self.installation
+            )
+
+            obj = RenderObject(body_model, data=instance, override_texture=body_texture)
+
+            head_hook = self.model(body_model).find("headhook")
+            if head_model and head_hook:
+                head_obj = RenderObject(head_model, override_texture=head_texture)
+                head_obj.set_transform(head_hook.global_transform())
+                obj.children.append(head_obj)
+
+            rhand_hook = self.model(body_model).find("rhand")
+            if rhand_model and rhand_hook:
+                rhand_obj = RenderObject(rhand_model)
+                rhand_obj.set_transform(rhand_hook.global_transform())
+                obj.children.append(rhand_obj)
+
+            lhand_hook = self.model(body_model).find("lhand")
+            if lhand_model and lhand_hook:
+                lhand_obj = RenderObject(lhand_model)
+                lhand_obj.set_transform(lhand_hook.global_transform())
+                obj.children.append(lhand_obj)
+
+            if head_hook is None:
+                mask_hook = self.model(body_model).find("gogglehook")
+            elif head_model:
+                mask_hook = self.model(head_model).find("gogglehook")
+            if mask_model and mask_hook:
+                mask_obj = RenderObject(mask_model)
+                mask_obj.set_transform(mask_hook.global_transform())
+                if head_hook is None:
+                    obj.children.append(mask_obj)
+                elif head_obj is not None:
+                    head_obj.children.append(mask_obj)
+
+        except Exception as e:
+            print(e)
+            # If failed to load creature models, use the unknown model instead
+            obj = RenderObject("unknown", data=instance)
+
+        return obj
 
     def buildCache(self, clearCache: bool = False) -> None:
         if self.module is None:
@@ -172,23 +238,7 @@ class Scene:
 
         for creature in self.git.creatures:
             if creature not in self.objects:
-                try:
-                    utc = self.module.creature(creature.resref.get()).resource()
-                    body_model = self.table_creatures.get_row(utc.appearance_id).get_string("race")
-                    obj = RenderObject(body_model, vec3(), vec3(), data=creature)
-
-                    head_str = self.table_creatures.get_row(utc.appearance_id).get_string("normalhead")
-                    if head_str:
-                        head_id = int(head_str)
-                        head_model = self.table_heads.get_row(head_id).get_string("head")
-                        head_obj = RenderObject(head_model)
-                        head_obj.set_transform(self.model(body_model).find("headhook").global_transform())
-                        obj.children.append(head_obj)
-                except Exception:
-                    # If failed to load creature models, use an empty model instead
-                    obj = RenderObject("unknown", vec3(), vec3(), data=creature)
-
-                self.objects[creature] = obj
+                self.objects[creature] = self.getCreatureRenderObject(creature)
 
             self.objects[creature].set_position(creature.position.x, creature.position.y, creature.position.z)
             self.objects[creature].set_rotation(0, 0, creature.bearing)
@@ -343,9 +393,11 @@ class Scene:
             return
 
         model = self.model(obj.model)
-        model.draw(shader, transform * obj.transform())
+        transform = transform * obj.transform()
+        model.draw(shader, transform, override_texture=obj.override_texture)
+
         for child in obj.children:
-            self._render_object(shader, child, obj.transform())
+            self._render_object(shader, child, transform)
 
     def picker_render(self) -> None:
         glClearColor(1.0, 1.0, 1.0, 1.0)
@@ -438,8 +490,10 @@ class Scene:
     def texture(self, name: str) -> Texture:
         if name not in self.textures:
             try:
+                tpc = None
                 # Check the textures linked to the module first
-                tpc = self.module.texture(name).resource() if self.module.texture(name) is not None else None
+                if self.module is not None:
+                    tpc = self.module.texture(name).resource() if self.module.texture(name) is not None else None
                 # Otherwise just search through all relevant game files
                 tpc = self.installation.texture(name, [SearchLocation.OVERRIDE, SearchLocation.TEXTURES_TPA,
                                                        SearchLocation.CHITIN]) if tpc is None else tpc
@@ -486,8 +540,9 @@ class Scene:
                 mdl_data = UNKNOWN_MDL_DATA
                 mdx_data = UNKNOWN_MDX_DATA
             elif self.installation is not None:
-                mdl_search = self.installation.resource(name, ResourceType.MDL, SEARCH_ORDER, capsules=self.module.capsules())
-                mdx_search = self.installation.resource(name, ResourceType.MDX, SEARCH_ORDER, capsules=self.module.capsules())
+                capsules = [] if self.module is None else self.module.capsules()
+                mdl_search = self.installation.resource(name, ResourceType.MDL, SEARCH_ORDER, capsules=capsules)
+                mdx_search = self.installation.resource(name, ResourceType.MDX, SEARCH_ORDER, capsules=capsules)
                 if mdl_search and mdx_search:
                     mdl_data = mdl_search.data
                     mdx_data = mdx_search.data
@@ -515,7 +570,7 @@ class Scene:
 
 class RenderObject:
     def __init__(self, model: str, position: vec3 = None, rotation: vec3 = None, *, data: Any = None,
-                 genBoundary: Callable[[], Boundary] = None):
+                 genBoundary: Callable[[], Boundary] = None, override_texture: Optional[str] = None):
         self.model: str = model
         self.children: List[RenderObject] = []
         self._transform: mat4 = mat4()
@@ -525,6 +580,7 @@ class RenderObject:
         self._boundary: Optional[Boundary] = None
         self.genBoundary: Optional[Callable[[], Boundary]] = genBoundary
         self.data: Any = data
+        self.override_texture: Optional[str] = override_texture
 
         self._recalc_transform()
 
@@ -538,14 +594,15 @@ class RenderObject:
         self._rotation = glm.eulerAngles(rotation)
 
     def _recalc_transform(self) -> None:
-        self._transform = glm.translate(mat4(), self._position)
+        print("recalc", self.model)
+        self._transform = mat4() * glm.translate(self._position)
         self._transform = self._transform * glm.mat4_cast(quat(self._rotation))
 
     def position(self) -> vec3:
         return copy(self._position)
 
     def set_position(self, x: float, y: float, z: float) -> None:
-        if self._position.x == x and self._position.y == y and self._position.z == z:
+        if math.isclose(self._position.x, x, rel_tol=1e-03) and math.isclose(self._position.y, y, rel_tol=1e-03) and math.isclose(self._position.z, z, rel_tol=1e-03):
             return
 
         self._position = vec3(x, y, z)
@@ -555,7 +612,7 @@ class RenderObject:
         return copy(self._rotation)
 
     def set_rotation(self, x: float, y: float, z: float) -> None:
-        if self._rotation.x == x and self._rotation.y == y and self._rotation.z == z:
+        if math.isclose(self._rotation.x, x, rel_tol=1e-03) and math.isclose(self._rotation.y, y, rel_tol=1e-03) and math.isclose(self._rotation.z, z, rel_tol=1e-03):
             return
 
         self._rotation = vec3(x, y, z)
@@ -572,6 +629,11 @@ class RenderObject:
             self._cube = Cube(scene, min_point, max_point)
         return self._cube
 
+    def radius(self, scene: Scene) -> float:
+        cube = self.cube(scene)
+        return max(abs(cube.min_point.x), abs(cube.min_point.y), abs(cube.min_point.z),
+                   abs(cube.max_point.x), abs(cube.max_point.y), abs(cube.max_point.z))
+
     def _cube_rec(self, scene: Scene, transform: mat4, obj: RenderObject, min_point: vec3, max_point: vec3) -> None:
         obj_min, obj_max = scene.model(obj.model).box()
         obj_min = transform * obj_min
@@ -618,7 +680,7 @@ class Camera:
         y += math.sin(self.yaw) * math.cos(self.pitch - math.pi/2) * self.distance
         z += math.sin(self.pitch - math.pi/2) * self.distance
 
-        camera = glm.translate(mat4(), vec3(x, y, z))
+        camera = mat4() * glm.translate(vec3(x, y, z))
         camera = glm.rotate(camera, self.yaw + math.pi/2, up)
         camera = glm.rotate(camera, math.pi - self.pitch, pitch)
         view = glm.inverse(camera)
