@@ -11,7 +11,7 @@ from OpenGL.raw.GL.VERSION.GL_1_0 import GL_TEXTURE_2D, glTexParameteri, GL_RGB,
     GL_RGBA, GL_NEAREST_MIPMAP_LINEAR
 from OpenGL.raw.GL.VERSION.GL_1_1 import glBindTexture
 from OpenGL.raw.GL.VERSION.GL_1_3 import glCompressedTexImage2D
-from OpenGL.raw.GL.VERSION.GL_2_0 import GL_VERTEX_SHADER, GL_FRAGMENT_SHADER, glUseProgram
+from OpenGL.raw.GL.VERSION.GL_2_0 import GL_VERTEX_SHADER, GL_FRAGMENT_SHADER, glUseProgram, glUniform1i
 from glm import mat4, vec4, vec3
 from pykotor.resource.formats.tpc import TPC, TPCTextureFormat
 
@@ -49,12 +49,18 @@ out vec4 FragColor;
 
 layout(binding = 0) uniform sampler2D diffuse;
 layout(binding = 1) uniform sampler2D lightmap;
+uniform int enableLightmap;
 
 void main()
 {
     vec4 diffuseColor = texture(diffuse, diffuse_uv);
     vec4 lightmapColor = texture(lightmap, lightmap_uv);
-    FragColor = mix(diffuseColor, lightmapColor, 0.5);
+    
+    if (enableLightmap == 1) {
+        FragColor = mix(diffuseColor, lightmapColor, 0.5);
+    } else {
+        FragColor = diffuseColor;
+    }
 }
 """
 
@@ -119,59 +125,6 @@ void main()
 """
 
 
-TERRAIN_VSHADER = """
-#version 330 core
-
-layout (location = 0) in vec3 flags;
-layout (location = 1) in vec3 position;
-layout (location = 2) in vec3 normal;
-layout (location = 3) in vec3 uv;
-layout (location = 4) in vec3 uv2;
-
-out vec2 diffuse_uv;
-out vec2 lightmap_uv;
-out float dist;
-
-uniform mat4 model;
-uniform mat4 view;
-uniform mat4 projection;
-uniform vec3 cursor;
-
-void main()
-{
-    gl_Position = projection * view * model * vec4(position, 1.0);
-    
-    diffuse_uv = vec2(uv.x, uv.y);
-    lightmap_uv = vec2(uv2.x, uv2.y);
-    
-    dist = (10 - distance(vec2(position), vec2(cursor)))/10;
-    dist = clamp(dist, 0.5, 1);
-}
-"""
-
-
-TERRAIN_FSHADER = """
-#version 420
-in vec2 diffuse_uv;
-in vec2 lightmap_uv;
-in float dist;
-
-out vec4 FragColor;
-
-layout(binding = 0) uniform sampler2D diffuse;
-layout(binding = 1) uniform sampler2D lightmap;
-
-void main()
-{
-    //float distance = max(0, 1-distance(vec2(gl_FragCoord), vec2(cursor))/100);
-    //float dist = (1000 - distance(vec2(gl_FragCoord), vec2(cursor)))/1000;
-    
-    vec4 diffuseColor = texture(diffuse, diffuse_uv);
-    FragColor = diffuseColor * vec4(dist, dist, dist, 0);
-}
-"""
-
-
 class Shader:
     def __init__(self, vshader: str, fshader: str):
         vertex_shader = shaders.compileShader(vshader, GL_VERTEX_SHADER)
@@ -193,6 +146,9 @@ class Shader:
     def set_vector3(self, uniform: str, vector: vec3):
         glUniform3fv(self.uniform(uniform), 1, glm.value_ptr(vector))
 
+    def set_bool(self, uniform: str, boolean: bool):
+        glUniform1i(self.uniform(uniform), boolean)
+
 
 class Texture:
     def __init__(self, tex_id: int):
