@@ -1,4 +1,4 @@
-from pykotor.resource.formats.mdl.mdl_data import MDL
+from pykotor.resource.formats.mdl.mdl_data import MDL, MDLNode, MDLMesh
 from pykotor.resource.formats.mdl.io_mdl import MDLBinaryReader, MDLBinaryWriter
 from pykotor.resource.formats.mdl.io_mdl_ascii import MDLAsciiReader, MDLAsciiWriter
 from pykotor.resource.formats.mdl.mdl_auto import write_mdl, read_mdl
