@@ -1,6 +1,6 @@
 import struct
 from copy import deepcopy
-from typing import Dict, List
+from typing import Dict, List, NamedTuple
 
 from pykotor.common.geometry import Vector3, Vector4
 from pykotor.common.misc import Game
@@ -59,6 +59,11 @@ _NODE_TYPE_EMITTER = 4
 _EMITTER_HEADER_SIZE = 224
 
 
+class MDLMDXTuple(NamedTuple):
+    mdl: bytes
+    mdx: bytes
+
+
 def rename(
         data: bytes,
         name: str
@@ -649,3 +654,170 @@ def transform(
     data += struct.pack("fffff", 0.0, *orientation)
 
     return struct.pack("III", 0, len(data), mdx_size) + data
+
+
+def flip(
+        mdl_data: bytes,
+        mdx_data: bytes,
+        flip_x: bool,
+        flip_y: bool
+) -> MDLMDXTuple:
+    """
+    Returns the given MDL and MDX data with the vertices flipped along the specified axes.
+
+    Args:
+        mdl_data: The raw MDL data.
+        mdx_data: The raw MDX data.
+        flip_x: Flip the vertices across the X-axis.
+        flip_y: Flip the vertices across the Y-axis.
+
+    Returns:
+        The MDL and MDX data post-flip.
+    """
+
+    # If neither bools are set to True, no transformations need to be done and we can just return the original data
+    if not flip_x and not flip_y:
+        return MDLMDXTuple(mdl_data, mdx_data)
+
+    # The data we need to change:
+    #    1. The vertices stored in the MDL
+    #    2. The vertex positions, normals, stored in the MDX
+
+    # Trim the data to correct the offsets
+    mdl_start = mdl_data[:12]
+    mdl_data = bytearray(mdl_data[12:])
+    mdx_data = bytearray(mdx_data)
+
+    mdl_vertex_offsets = []  # This is a list of tuples: (count, offset)
+    mdx_vertex_offsets = []  # This is a list of tuples: (count, offset, stride, position)
+    mdx_normal_offsets = []  # This is a list of tuples: (count, offset, stride, position)
+    elements_offsets = []  # This is a list of tuples: (count, offset)
+    faces_offsets = []  # This is a list of tuples: (count, offset)
+    with BinaryReader.from_bytes(mdl_data) as reader:
+        reader.seek(168)
+        root_offset = reader.read_uint32()
+
+        nodes = [root_offset]
+        while nodes:
+            node_offset = nodes.pop()
+            reader.seek(node_offset)
+            node_id = reader.read_uint32()
+
+            mdl_vertex_offsets.append((1, node_offset+16))
+
+            # Need to determine the location of the position controller
+            reader.seek(node_offset + 56)
+            controllers_offset = reader.read_uint32()
+            controllers_count = reader.read_uint32()
+
+            reader.seek(node_offset + 68)
+            controller_datas_offset = reader.read_uint32()
+            controller_datas_count = reader.read_uint32()
+
+            for i in range(controllers_count):
+                reader.seek(controllers_offset + i*16)
+                controller_type = reader.read_uint32()
+                if controller_type == 8:
+                    reader.skip(6)
+                    data_offset = reader.read_uint16()
+                    mdl_vertex_offsets.append((1, controller_datas_offset + data_offset*4))
+
+            reader.seek(node_offset + 44)
+            child_offsets_offset = reader.read_uint32()
+            child_offsets_count = reader.read_uint32()
+
+            reader.seek(child_offsets_offset)
+            for i in range(child_offsets_count):
+                nodes.append(reader.read_uint32())
+
+            if node_id & 32:
+                reader.seek(node_offset+80)
+                fp = reader.read_uint32()
+                tsl = False if fp in [_MESH_FP0_K1, _SKIN_FP0_K1, _DANGLY_FP0_K2, _AABB_FP0_K1, _SABER_FP0_K1] else True
+
+                reader.seek(node_offset+80+8)
+                faces_offset = reader.read_uint32()
+                faces_count = reader.read_uint32()
+                faces_offsets.append((faces_count, faces_offset))
+
+                reader.seek(node_offset+80+188)
+                offset_to_elements_offset = reader.read_uint32()
+                reader.seek(offset_to_elements_offset)
+                elements_offset = reader.read_uint32()
+                elements_offsets.append((faces_count, elements_offset))
+
+                reader.seek(node_offset+80+304)
+                vertex_count = reader.read_uint16()
+                reader.seek(node_offset+80+336 if tsl else node_offset+80+328)
+                vertex_offset = reader.read_uint32()
+                mdl_vertex_offsets.append((vertex_count, vertex_offset))
+
+                reader.seek(node_offset+80+252)
+                mdx_stride = reader.read_uint32()
+                mdx_bitmap = reader.read_uint32()
+                reader.seek(node_offset+80+260)
+                mdx_offset_pos = reader.read_uint32()
+                mdx_offset_norm = reader.read_uint32()
+                reader.seek(node_offset+80+332 if tsl else node_offset+80+324)
+                mdx_start = reader.read_uint32()
+                mdx_vertex_offsets.append((vertex_count, mdx_start, mdx_stride, mdx_offset_pos))
+                mdx_normal_offsets.append((vertex_count, mdx_start, mdx_stride, mdx_offset_norm))
+
+    # Fix vertex order
+    if flip_x != flip_y:
+        for count, start_offset in elements_offsets:
+            for i in range(count):
+                offset = start_offset + i*6
+                v1 = struct.unpack("H", mdl_data[offset:offset+2])[0]
+                v2 = struct.unpack("H", mdl_data[offset+2:offset+4])[0]
+                v3 = struct.unpack("H", mdl_data[offset+4:offset+6])[0]
+                mdl_data[offset:offset+2] = struct.pack("H", v1)
+                mdl_data[offset+2:offset+4] = struct.pack("H", v3)
+                mdl_data[offset+4:offset+6] = struct.pack("H", v2)
+        for count, start_offset in faces_offsets:
+            for i in range(count):
+                offset = start_offset + i*32 + 26
+                v1 = struct.unpack("H", mdl_data[offset:offset+2])[0]
+                v2 = struct.unpack("H", mdl_data[offset+2:offset+4])[0]
+                v3 = struct.unpack("H", mdl_data[offset+4:offset+6])[0]
+                mdl_data[offset:offset+2] = struct.pack("H", v1)
+                mdl_data[offset+2:offset+4] = struct.pack("H", v3)
+                mdl_data[offset+4:offset+6] = struct.pack("H", v2)
+
+    # Update the MDL vertices
+    for count, start_offset in mdl_vertex_offsets:
+        for i in range(count):
+            offset = start_offset + i*12
+            if flip_x:
+                x = struct.unpack("f", mdl_data[offset:offset+4])[0]
+                mdl_data[offset:offset+4] = struct.pack("f", -x)
+            if flip_y:
+                y = struct.unpack("f", mdl_data[offset+4:offset+8])[0]
+                mdl_data[offset+4:offset+8] = struct.pack("f", -y)
+
+    # Update the MDX vertices
+    for count, start_offset, stride, position in mdx_vertex_offsets:
+        for i in range(count):
+            offset = start_offset + i*stride + position
+            if flip_x:
+                x = struct.unpack("f", mdx_data[offset:offset+4])[0]
+                mdx_data[offset:offset+4] = struct.pack("f", -x)
+            if flip_y:
+                y = struct.unpack("f", mdx_data[offset+4:offset+8])[0]
+                mdx_data[offset+4:offset+8] = struct.pack("f", -y)
+
+    # Update the MDX normals
+    for count, start_offset, stride, position in mdx_normal_offsets:
+        for i in range(count):
+            offset = start_offset + i * stride + position
+            if flip_x:
+                x = struct.unpack("f", mdx_data[offset:offset + 4])[0]
+                mdx_data[offset:offset + 4] = struct.pack("f", -x)
+            if flip_y:
+                y = struct.unpack("f", mdx_data[offset + 4:offset + 8])[0]
+                mdx_data[offset + 4:offset + 8] = struct.pack("f", -y)
+
+    # Readd the first 12 bytes
+    mdl_data = mdl_start + mdl_data
+
+    return MDLMDXTuple(mdl_data, mdx_data)
